local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)

NTWatch = {}

function NTWatch:New(parent)
    local ret = CloneTable(NTWatch)
    ret.parent = parent
    if ret.Initialize then
        ret:Initialize()
    end
    return ret
end

function NTWatch:GetRollStatistics()
    local db = self.parent.db
    return GetSubtable(db.char, "RollStatistics")
end

function NTWatch:GetRollHistory()
    local db = self.parent.db
    return GetSubtable(db.char, "RollHistory")
end

function NTWatch:AddRollStatistics(player, choice)
    local db = self.parent.db
    local stat = GetSubtable(db.char, "RollStatistics")

    local n, data, found
    for n, data in pairs(stat) do
        if data.player == player then
            found = true
            data[choice] = data[choice] + 1
            break
        end
    end
    if not found then
        table.insert(stat, { player = player, win = 0, need = 0, greed = 0, disenchant = 0, pass = 0, [choice] = 1 })
    end

    self.parent.gui:UpdateRollStatistics()
end

function NTWatch:AddRollHistory(item, res, choice, player, roll)
    local db = self.parent.db
    local hist = GetSubtable(db.char, "RollHistory")

    table.insert(hist, 1, { item = item, res = res, choice = choice, player = player, roll = roll })

    self.parent.gui:UpdateRollHistory()
end

function NTWatch:ClearRollStatistics()
    local db = self.parent.db

    db.char.RollStatistics = {}
    db.char.RollHistory = {}

    self.parent.gui:UpdateRollStatistics()
    self.parent.gui:UpdateRollHistory()
end

function NTWatch:EventAllPassed(item)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item)

    db.char.RollResults[item] = nil

    self:AddRollHistory(item, res, "pass")

    self.parent.reporter:EventAllPassed(item)
end

function NTWatch:EventChose(item, player, choice)
    local db = self.parent.db
    GetSubtable(db.char, "RollResults", item, player).choice = choice

    self:AddRollStatistics(player, choice)

    if choice == "need" and player ~= UnitName("player") then
        self.parent.reporter:EventChoseNeed(item, player)
    end
end

function NTWatch:EventRolled(item, player, choice, roll)
    local db = self.parent.db

    local res = GetSubtable(db.char, "RollResults", item, player)
    res.choice, res.roll = choice, roll
end

function NTWatch:EventWon(item, player)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item, player)

    self:EventWonNoSpam(item, player, res.choice or "unknown", res.roll or "?")
end

function NTWatch:EventWonNoSpam(item, player, choice, roll)
    local db = self.parent.db
    local res = GetSubtable(db.char, "RollResults", item)

    db.char.RollResults[item] = nil

    self:AddRollHistory(item, res, choice, player, roll)
    self:AddRollStatistics(player, "win")

    self.parent.reporter:EventWon(item, player, choice, roll)
end

function NTWatch:EventJoinGroup()
    self.parent.db.char.RollResults = {}
    self.parent.gui:ConfirmClearStatistics()
end

function NTWatch:EventLeaveGroup()
    self.parent.db.char.RollResults = {}
end

function NTWatch:EventEnterInstance()
    self.parent.gui:ConfirmClearStatistics()
end
